<?php
// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// | Copyright (c) 2003 Mike Hommey                                       |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Mike Hommey <mh@glandium.org>                                |
// +----------------------------------------------------------------------+
//
// $Id: request.php,v 1.6 2003/07/23 17:35:39 glandium Exp $

include_once(PAPAAN_BASE.'/components/generator.php');

/**
* This class returns xml from request headers
*
* @author   Mike Hommey <mh@glandium.org>
* @version  $Id: request.php,v 1.6 2003/07/23 17:35:39 glandium Exp $
* @package  papaan
*/
class generator_request extends generator {


    /**
    * Constructor, does nothing at the moment
    */
    function generator_request (&$sitemap, $config) {
        $this->generator($sitemap, $config);
    }

    /**
    * Initiator, called after construction of object
    *
    *  This method will be called in the start element with the attributes from this element
    *
    *  As we just call the parent init method, it's not really needed, 
    *   it's just here for reference
    *
    *  @param $attribs array    associative array with element attributes
    *  @access public
    */
    function init($attribs)
    {
        parent::init($attribs);
    }    
    
    /**
    * generates an xml-DomDocument out of the xml-file
    *
    * @access public
    * @returns object DomDocument XML-Document
    */
    function DomStart(&$xml)
    {
	$src = $this->getAttrib('src');
	$xml = '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
	/* need to add target and source */
	$xml .= '<request target="' . htmlspecialchars($_SERVER['REQUEST_URI']) . '" ' . ($src ? 'source="'.$src.'" ':'').'xmlns="http://xml.apache.org/cocoon/requestgenerator/2.0">' . "\n";
	$xml .= '<requestHeaders>' . "\n";
	foreach ($_SERVER as $key => $value) {
		if (strtolower(substr($key,0,5)) == 'http_') {
			$xml .= '<header name="'. strtr(strtolower(substr($key,5)),'_','-') . '">' . $value . '</header>' . "\n";
		}
	}
	$xml .= '</requestHeaders>' . "\n";
	$xml .= '<requestParameters>' . "\n";
	foreach ($this->sitemap->request->get_parameter_names() as $key) {
		$xml .= '<parameter name="'. $key . '"><value>' . $this->sitemap->request->get_parameter($key) . '</value></parameter>' . "\n";
	}
	$xml .= '</requestParameters>' . "\n";
	if ($this->getParameter('generate-attributes') == 'true') {
		$xml .= '<requestAttributes>';
		foreach ($this->sitemap->request->get_attribute_names() as $key) {
			$xml .= '<attribute name="'. $key . '"><value>' . $this->sitemap->request->get_attribute($key) . '</value></attribute>' . "\n";
		}
		$xml .= '</requestAttributes>';
	}
	$xml .= '<configurationParameters>';
	foreach (array_keys($this->params) as $key) {
		$xml .= '<parameter name="'. $key . '">' . $this->getParameter($key) . '</parameter>' . "\n";
	}
	$xml .= '</configurationParameters>';
	$xml .= '</request>';
	$this->sitemap->var2XMLObject($xml);
        return True;
    }

    
    /* CACHING STUFF */

    /**
     * Generate cacheKey
     *
     * Calls the method inherited from 'Component'
     *
     * @param   array  attributes
     * @param   int    last cacheKey
     * @see     generateKeyDefault()
     */
    function generateKey($attribs, $keyBefore){
        return($this->generateKeyDefault($attribs, $keyBefore));
    }

    /** Generate validityObject  
     *
     * This is common to all "readers", you'll find the same code there.
     * I'm thinking about making a method in the class component named generateValidityFile() or alike
     * instead of having the same code everywhere..
     *
     * @author Hannes Gassert <hannes.gassert@unifr.ch>
     * @see  checkvalidity()
     * @return  array  $validityObject contains the components attributes plus file modification time and time of last access.
     */
    function generateValidity(){
        $validityObject = $this->attribs;
        $src = $this->getAttrib('src');
        $validityObject['filemtime'] = filemtime($src);
        $validityObject['fileatime'] = fileatime($src);
        return($validityObject);
    }

    /**
     * Check validity of a validityObject from cache
     *
     * This implements only the most simple form: If there's no fresher version, take that from cache.
     * I guess we'll need some more refined criteria..
     *
     * @return  bool  true if the validityObject indicates that the cached version can be used, false otherwise.
     * @param   object  validityObject
     */
    function checkValidity($validityObject){
        return(isset($validityObject['src'])       &&
               isset($validityObject['filemtime']) &&
               file_exists($validityObject['src']) &&
               ($validityObject['filemtime'] == filemtime($validityObject['src'])));
    }

}


?>
