<?php

// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Christian Stocker <chregu@bitflux.ch>                        |
// +----------------------------------------------------------------------+
//
// $Id: papaan.php,v 1.19 2003/07/24 18:03:51 glandium Exp $

define('PAPAAN_VERSION', '0.4.1');

define('PAPAAN_STRING', 'Papaan v'.PAPAAN_VERSION);

define('PAPAAN_BASE', dirname(__FILE__));

include_once(PAPAAN_BASE.'/sitemap/sitemap.php');

/**
* We have different error levels in papaan, it's much like
*  the PHP error handling. With PEAR::raiseError you can
*  declare how severe your error is.
**/
/**
* Won't produce any output ever
* @const PAPAAN_ERROR_SILENT
* @access public
*/
define('PAPAAN_ERROR_SILENT',     1);
/**
* Errors, which are not that grave
* Errors with that level won't disturb papaan. They are more
*  of an informational level
* @const PAPAAN_ERROR_NOTICE
* @access public
*/
define('PAPAAN_ERROR_NOTICE',     2);
/**
* Errors, which should be avoided, but the script is able to run further
* Errors with that level could have bad effects on papaan, but normally don't
* @const PAPAAN_ERROR_WARNING
* @access public
*/
define('PAPAAN_ERROR_WARNING',     E_USER_WARNING);
/**
* Errors, which stop the execution of papaan
* Errors with that level have severe impact on papaan and the script will be stoped
* @const PAPAAN_ERROR_WARNING
* @access public
*/
define('PAPAAN_ERROR_FATAL',       E_USER_ERROR);

/**
* Can be used for old deprecated functions..
* @const PAPAAN_ERROR_DEPRECATED
* @access public
*/
define('PAPAAN_ERROR_DEPRECATED',       16);

/**
* No Error-reporting at all....
* @const PAPAAN_ERROR_NONE
* @access public
*/
define('PAPAAN_ERROR_NONE', 0);
/**
* Every error-reporting level
* @const PAPAAN_ERROR_NONE
* @access public
*/
define('PAPAAN_ERROR_ALL', 31);
/**
* Default error-reporting level
* At the moment = PAPAAN_ERROR_ALL
* @const PAPAAN_ERROR_NONE
* @access public
*/
define('PAPAAN_ERROR_DEFAULT', PAPAAN_ERROR_ALL);

/**
* Class for doing everything in papaan :)
*
* @author   Christian Stocker <chregu@bitflux.ch>
* @version  $Id: papaan.php,v 1.19 2003/07/24 18:03:51 glandium Exp $
* @package  papaan2
*/
class papaan {

	/**
	* The Directory, where the cached sitemaps should be saved
	*
	* @var string
	*/
	var $cacheDir = 'cache';
	var $uri = '';

	/**
    * Constructor
    *
	* If $sitemapFile (or more) is provided, the method run() is immediately called.
	*
    * @param string $sitemapFile the location of sitemap.xml, can be relativ
	* @param string $uri the uri of the call, optional (takes _SERVER['REQUEST_URI'] then)
	* @param array $options class variable options
	* @see run()
    * @access public
	*/
    function papaan ($sitemapFile = null, $options = array()) {

	error_reporting(E_ALL ^ E_NOTICE);
	set_error_handler("myErrorHandler");

	if (!is_array($options)) {
		$this->Error('papaan','error','Invalid options passed to papaan constructor','You are probably using the old constructor way, passing the uri as second argument. This is deprecated. If you want to specify uri by yourself, put it in the options array. See documentation for further details.');
	}

        if ((isset($options['outputCache']) && $options['outputCache']) || (isset($options['outputCacheCallback']) && $options['outputCacheCallback'] && call_user_func($options['outputCacheCallback'])) )
        {
            include_once(PAPAAN_BASE.'/sitemap/outputcache.php');
            $oc = new OutputCache();
            $oc->start();
            $options['outputCache'] = true;

        } else {
            $options['outputCache'] = false;
        }

	$this->setOptions($options);
	$request = new request(isset($options['uri'])?$this->uri:null);
	$this->uri = $request->get_uri();
	$options['request'] = $request;
	if (!isset($options['cacheDir'])) {
		$options['cacheDir'] = $this->cacheDir;
	}
		if ($sitemapFile !== null)
		{
			$sm = $this->run(realpath($sitemapFile),$options);
		}

        if ($sm->outputCache)
        {
            if (! isset($options['outputCacheExpire'])) {
                $options['outputCacheExpire'] = 3600;
            }
        	$oc->end($sm, $options['outputCacheExpire']);
        }
	}

	/**
	* runs the sitemap
	*
	* More docu, please
	*
    * @param string $sitemapFile the location of sitemap.xml, can be relativ
	* @param string $uri the uri of the call, optional (takes _SERVER['REQUEST_URI'] then)
	* @param array $options class variable options
	* @see papaan(), setOptions
    * @access public
	*/
	function run($sitemapFile, $options = array()) {
		$sm = new sitemap($sitemapFile,$this->uri,$options);
		$sm->runSitemap();
		$sm->output();
		return $sm;
	}


	/**
	* Sets class options
	*
	* Sets options for a class. the array has to be an associative
	* array, where the key is a class-variable and the value is the
	* value. With this method, you can't set class-variable, which
	* were not defined before
	*
	* @param array $options associative array
    * @access public
	*/
	function setOptions($options) {
		foreach($options as $key => $value) {
			if (isset($this->$key)) {
				$this->$key = $value;
			}
		}
	}

	/**
	* Strips the starting xml processing instruction from a xml file
	*
	* This is needed, if we have short_open_tag = On, because then php
	* chokes on it. We don't need it if it's = Off.
	*
	* @param string $xml The XML Document as a string
	* @access public
	* @return string The XML without the PI
	*/
	function stripXmlPI($xml)
	{
		return preg_replace('#^<\?xml[^>]*\?>\s*#','',$xml);;
	}

	/**
	* Save a file to the filesystem
	*
	* @param string $content The content of the file
	* @param string $filename The filename
	* @return bool
	* @access public
	*/
	function saveFile($content,$filename)
	{
		$fd = fopen($filename,'w');
		if (!$fd) {
			papaan::raiseError('Could not save '.$filename.' (In '. __FILE__ .':'.__LINE__.')',PAPAAN_ERROR_FATAL);
			return false;
		}
		fwrite($fd,$content);
		fclose($fd);
		return True;
	}


    function Error($component_name, $type, $message, $description = null) {

	if (! $description) {
		$description = $message;
	}
	if ($type == 'resource-not-found') {
		if (!headers_sent()) {
			header('HTTP/1.0 404 Not Found');
			header('Status: 404 Not Found');
		}	
		$title = 'Resource Not Found';
	} else {
		if (!headers_sent()) {
			header('HTTP/1.0 500 Internal Server Error');
			header('Status: 500 Internal Server Error');
		}
		$title = 'Internal Server Error';
	}
	print '<?xml version="1.0" encoding="ISO-8859-1"?>';
?>

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title><?php echo $title; ?></title>
<style type="text/css">
html { font-family: sans-serif; line-height: 1.3em }
h1 { font-size: 1.3em; border-bottom: 1px solid black; }
dt { font-weight: bold }
dd { margin-left: 8em; margin-top: -1.3em; margin-bottom: .5em; }
</style>
</head>
<body>
<h1>Papaan - <?php echo $title; ?></h1>
<dl>
<dt>type</dt>
<dd><?php echo $type; ?></dd>
<dt>message</dt>
<dd><?php echo $message; ?></dd>
<dt>description</dt>
<dd><?php echo $description; ?></dd>
<dt>sender</dt>
<dd><?php echo $component_name; ?></dd>
<!-- Add stack trace if necessary -->
<?php if (isset($this->uri)) { ?>
<dt>request-uri</dt>
<dd>/<?php echo $this->uri; ?></dd>
<?php } ?>
</dl>
</body>
</html>
<?php
	exit;
    }

    function raiseError($msg, $code = null, $file = null, $line = null, $mode = null) {
    	myErrorHandler($code, $msg, $file, $line);
    }
}

    function myErrorHandler ($code = null, $msg, $file = null, $line = null) {
	if ($code == E_NOTICE) {
		return;
	}
	$description = $msg;
	if ($file) {
		if ($line) {
			$description .= ' on line '.$line.' of file ' .$file;
		} else {
			$description .= ' in file '.$file;
		}
	}
	switch ($code) {
	case E_ERROR:
	case PAPAAN_ERROR_FATAL:
		$type = 'fatal';
		break;
	case E_WARNING:
	case PAPAAN_ERROR_WARNING:
		$type = 'warning';
		break;
	default:
		$type = 'error';
		break;
	}
	if ($code == PAPAAN_ERROR_FATAL || $code == PAPAAN_ERROR_WARNING) {
		papaan::Error('papaan', 'fatal', $msg, $description);
	} else {
		papaan::Error('php', 'fatal', $msg, $description);
	}
    }

?>
